/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal;

import java.io.IOException;
import java.util.EventObject;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.conqat.ide.commons.ui.logging.LoggingUtils;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.command.CommandStackListener;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.Saveable;
import org.eclipse.ui.part.ViewPart;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.interfaces.ITopLevelElementChangeListener;
import org.fortiss.tooling.kernel.interfaces.ITopLevelElementContext;
import org.fortiss.tooling.kernel.services.IPersistencyService;
import org.fortiss.tooling.kernel.ui.internal.views.NavigatorViewPart;
import org.fortiss.tooling.kernel.ui.services.INavigatorService;
import org.fortiss.tooling.kernel.ui.util.EObjectSelectionUtils;

/**
 * This class implements the {@link INavigatorService} interface.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: B7F899DC2CFB74320D2F7A19A2D5FEBB
 */
public class NavigatorService implements INavigatorService,
		ITopLevelElementChangeListener, CommandStackListener {

	/** Stores the navigator view. */
	private NavigatorViewPart navigatorViewPart;

	/** Stores the {@link Saveable}s for the top-level elements. */
	private Map<ITopLevelElementContext, TopLevelElementSaveable> saveables = new HashMap<ITopLevelElementContext, NavigatorService.TopLevelElementSaveable>();

	/** Constructor. */
	public NavigatorService() {
		for (ITopLevelElementContext context : IPersistencyService.INSTANCE
				.getTopLevelElementContexts()) {
			context.addCommandStackListener(this);
			saveables.put(context, new TopLevelElementSaveable(context));
		}
		IPersistencyService.INSTANCE.addTopLevelElementListener(this);
	}

	/** Sets the navigator view. */
	public void setNavigatorViewPart(NavigatorViewPart part) {
		this.navigatorViewPart = part;
	}

	/** {@inheritDoc} */
	@Override
	public ViewPart getNavigatorView() {
		return navigatorViewPart;
	}

	/** {@inheritDoc} */
	@Override
	public ISelection getCurrentSelection() {
		return navigatorViewPart.getSite().getSelectionProvider()
				.getSelection();
	}

	/** {@inheritDoc} */
	@Override
	public void topLevelElementAdded(ITopLevelElementContext context) {
		context.addCommandStackListener(this);
		saveables.put(context, new TopLevelElementSaveable(context));
		navigatorViewPart.refresh();
	}

	/** {@inheritDoc} */
	@Override
	public void topLevelElementRemoved(ITopLevelElementContext context) {
		context.removeCommandStackListener(this);
		saveables.remove(context);
		navigatorViewPart.refresh();
	}

	/** {@inheritDoc} */
	@Override
	public void commandStackChanged(EventObject event) {
		navigatorViewPart.refresh();
	}

	/** {@inheritDoc} */
	@Override
	public Saveable[] getSaveables() {
		return saveables.values().toArray(new Saveable[saveables.size()]);
	}

	/** {@inheritDoc} */
	@Override
	public Saveable[] getActiveSaveables() {
		Set<Saveable> result = new HashSet<Saveable>();
		for (EObject selection : EObjectSelectionUtils
				.getCurrentSelectionEObjects()) {
			ITopLevelElementContext context = IPersistencyService.INSTANCE
					.getTopLevelElementContextFor(selection);
			if (saveables.get(context) != null) {
				result.add(saveables.get(context));
			}
		}
		return result.toArray(new Saveable[result.size()]);
	}

	/**
	 * Implementation of {@link Saveable}s for {@link ITopLevelElementContext}s.
	 */
	private static class TopLevelElementSaveable extends Saveable {

		/** Stores the {@link ITopLevelElementContext}. */
		public final ITopLevelElementContext context;

		/** Constructor. */
		public TopLevelElementSaveable(ITopLevelElementContext context) {
			this.context = context;
		}

		/** {@inheritDoc} */
		@Override
		public String getName() {
			return context.getSaveableName();
		}

		/** {@inheritDoc} */
		@Override
		public String getToolTipText() {
			return "";
		}

		/** {@inheritDoc} */
		@Override
		public ImageDescriptor getImageDescriptor() {
			return null;
		}

		/** {@inheritDoc} */
		@Override
		public void doSave(IProgressMonitor monitor) throws CoreException {
			try {
				context.doSave(monitor);
			} catch (IOException e) {
				LoggingUtils.error(ToolingKernelActivator.getDefault(),
						"Error during save of " + getName(), e);
			}
		}

		/** {@inheritDoc} */
		@Override
		public boolean isDirty() {
			return context.isDirty();
		}

		/** {@inheritDoc} */
		@Override
		public boolean equals(Object object) {
			return object instanceof TopLevelElementSaveable
					&& ((TopLevelElementSaveable) object).context == context;
		}

		/** {@inheritDoc} */
		@Override
		public int hashCode() {
			return context.hashCode();
		}
	}

	/** {@inheritDoc} */
	@Override
	public void revealModelElement(EObject modelElement) {
		navigatorViewPart.revealModelElement(modelElement);
	}
}
